package smartresource.shared;

import smartresource.core.*;

import java.util.*;

import javax.mail.*;
import javax.mail.internet.*;
import javax.activation.*;

public class EmailSenderBehavior extends ReusableAtomicBehavior{

       	long waitTill=0;
	Thread sendIt;

        MimeMessage msg;
	String receiver;

	boolean tried=false;

	public void action() {

	        if(sendIt!=null){ // control is in the sending thread
        		block();
        		return; 
        	}

   		if(tried && finished){
	  		if(success) print("Email to "+receiver+" is sent");
	  		else print("Sending email to "+receiver+" is failed!");
			return;   		
   		}

	    	if(waitTill!=0){ 
			long now=System.currentTimeMillis();
			if(now<waitTill){ // waiting till next attempt of resending
				block(waitTill-now);
				return;
			}
	    	}

		finished=false;
		tried=true;

		String smtp=getParameterValue("smtp");
		smtp.equals(""); //to cause exception if null

		receiver=getParameterValue("to");
		if(receiver==null || receiver.equals("")){
			success=false;
			return;
		}

		String sender=getParameterValue("from");
		if(sender==null) sender="no@addr.ess";
		
		String cc=getParameterValue("cc");
		String bcc=getParameterValue("bcc");
		String subject=getParameterValue("subject");
		if(subject==null) subject="";
		String message=getParameterValue("message");
		if(message==null) message="";
		String content=getParameterValue("contentType");
		if(content==null) content="text/plain";

		String attached=getParameterValue("attach");

		try{
			Properties props=System.getProperties();
			props.put("mail.smtp.host", smtp);
			Session session = Session.getDefaultInstance(props, null);
	
		        msg=new MimeMessage(session);
		 	msg.setFrom(new InternetAddress(sender));
		
		        InternetAddress[] address=InternetAddress.parse(receiver, false);
		   	msg.setRecipients(Message.RecipientType.TO, address);
			if(cc!=null){
				address=InternetAddress.parse(cc, false);
				msg.setRecipients(Message.RecipientType.CC, address);
		        }
		        if(bcc!=null){
		    		address=InternetAddress.parse(bcc, false);
		    		msg.setRecipients(Message.RecipientType.BCC, address);
	    		}

	    		msg.setSubject(subject);

			if(attached==null){
				msg.setText(message);
		  	}
	  		else{
	    			MimeBodyPart mbp1 = new MimeBodyPart();
	    			mbp1.setText(message);

			    	Multipart mp = new MimeMultipart();
	    			mp.addBodyPart(mbp1);

	    			StringTokenizer st= new StringTokenizer(attached,",");
	    			while(st.hasMoreTokens()){		
					String f=st.nextToken();
					MimeBodyPart mbpi = new MimeBodyPart();
        				String fil=f;
					FileDataSource fds = new FileDataSource(fil);
					mbpi.setDataHandler(new DataHandler(fds));
					mbpi.setFileName(fds.getName());
					mp.addBodyPart(mbpi);
	    			}

	    			msg.setContent(mp);
          		}

			msg.setHeader("X-Mailer", "SmartResource Platform EmailSenderBehavior");
			msg.setHeader("Content-Type", content);

	  		msg.setSentDate(new Date());
	    
			Runnable runIt = new SendEmail();
			sendIt = new Thread(runIt);
			sendIt.start();

			block();
		}
		catch(MessagingException ex){
			ex.printStackTrace();
			success=false;
			finished=true;
	  		print("Sending email to "+receiver+" is failed!");
		}
	}

	private class SendEmail implements Runnable{
		public void run() {
			try{
		  		Transport.send(msg);
				finished=true;
				sendIt=null;				
				restart();
		  	}
			catch(MessagingException ex){
			        String error=ex.getMessage();
				String wait=getParameterValue("waitOnFail");
			        if(wait==null || !(error.startsWith("451") || error.startsWith("452"))){
					ex.printStackTrace();
					success=false;
					finished=true;
				}
				else{
					long toWait=new Long(wait).longValue();
					long now=System.currentTimeMillis();
					waitTill=now+toWait;

			  		print("Email sending error. Waiting "+toWait/1000+" sec");
				}
				sendIt=null;				
				restart();
			}
		}
	}
}
