package smartresource.shared;

import smartresource.core.*;

import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;

public class RequestReceiverBehavior extends ReusableAtomicBehavior{

	String request_match="";
	String convID="";
	String perform="";
	String waitOnlyFirst="";
	String addBeliefs="";
	String maxWait="";

       	long waitTill=0;

	public void onStart(){
	        finished=false; //it is a cyclic behavior

		request_match=getParameterValue("matchRequest");
		convID=getParameterValue("matchConversationID");
		perform=getParameterValue("matchPerformative");

		addBeliefs=getParameterValue("addBeliefs");
		if(addBeliefs==null) addBeliefs="true";

		waitOnlyFirst=getParameterValue("waitOnlyFirst");
		if(waitOnlyFirst==null) waitOnlyFirst="false";

		maxWait=getParameterValue("maxWait");
		if(maxWait!=null){
			long toWait=new Long(maxWait).longValue();
			long now=System.currentTimeMillis();
			waitTill=now+toWait;
		}
	}

	public void action() {
	        MessageTemplate mt=MessageTemplate.MatchAll();
		if(request_match!=null) mt=MessageTemplate.and(mt,MessageTemplate.MatchContent(request_match));
		if(convID!=null) mt=MessageTemplate.and(mt,MessageTemplate.MatchConversationId(convID));
		if(perform!=null){
			int p=-1;
			if(perform.equals("request")) p=ACLMessage.REQUEST;
			else if(perform.equals("inform")) p=ACLMessage.INFORM;
			if(p!=-1) mt=MessageTemplate.and(mt,MessageTemplate.MatchPerformative(p));
		}

		ACLMessage msg = myAgent.receive(mt);
		if (msg != null){

			String requestParam=msg.getUserDefinedParameter("requestParameter");
			print("\""+msg.getContent()+((requestParam==null)?"":(" "+requestParam))+"\" from "+msg.getSender().getLocalName()+" received");

		        if(addBeliefs.equals("true")){ 
				long curr=System.currentTimeMillis();
				String requestId="request"+curr;
		
				addBelief(new SemanticStatement(requestId,"conversationID",msg.getConversationId()));
				addBelief(new SemanticStatement(requestId,"sender",msg.getSender().getLocalName()));
				addBelief(new SemanticStatement(requestId,"request",msg.getContent()));
				addBelief(new SemanticStatement(requestId,"parameter",msg.getUserDefinedParameter("requestParameter")));

				int p=msg.getPerformative();
				String perf="other";
				if(p==ACLMessage.REQUEST) perf="request";
				else if(p==ACLMessage.INFORM) perf="inform";
				addBelief(new SemanticStatement(requestId,"performative",perf));

				while(System.currentTimeMillis()==curr) myAgent.doWait(2);
			}

			if(waitOnlyFirst.equals("true")) finished=true;
			else onEnd(); // to perform addOnSuccess, removeOnSuccess, and wakeAgent
		
		}
		else {
		        if(maxWait!=null){
				long now=System.currentTimeMillis();
				if(now>=waitTill){
				        success=false;
					finished=true;
				}
				else block(waitTill-now);
			}
			else block();
		}
	}
}
